@@ -7,8 +7,8 @@
 // Media Patent License 1.0 was not distributed with this source code in the
 // PATENTS file, you can obtain it at www.aomedia.org/license/patent.
 
-use debug_unreachable::debug_unreachable;
-use rust_hawktracer::*;
+use std::alloc::handle_alloc_error;
+use std::iter::FusedIterator;
 use std::marker::PhantomData;
 use std::mem;
 use std::ops::{Index, IndexMut, Range};
@@ -20,7 +20,6 @@ use std::{
     fmt::{Debug, Display, Formatter},
     usize,
 };
-use std::{iter::FusedIterator, ops::DerefMut};
 
 use crate::math::*;
 use crate::pixel::*;
@@ -196,12 +195,17 @@ impl<T: Pixel> PlaneData<T> {
     }
 
     unsafe fn layout(len: usize) -> Layout {
-        Layout::from_size_align_unchecked(len * mem::size_of::<T>(), 1 << Self::DATA_ALIGNMENT_LOG2)
+        Layout::from_size_align(len * mem::size_of::<T>(), 1 << Self::DATA_ALIGNMENT_LOG2)
+            .expect("layout size too large")
     }
 
     unsafe fn new_uninitialized(len: usize) -> Self {
         let ptr = {
-            let ptr = alloc(Self::layout(len)) as *mut T;
+            let layout = Self::layout(len);
+            let ptr = alloc(layout) as *mut T;
+            if ptr.is_null() {
+                handle_alloc_error(layout);
+            }
             std::ptr::NonNull::new_unchecked(ptr)
         };
 
@@ -595,6 +599,7 @@ impl<T: Pixel> Plane<T> {
                 *dst = T::cast_from(avg);
             }
         }
+
         new.pad(frame_width, frame_height);
         new
     }
@@ -613,9 +618,22 @@ impl<T: Pixel> Plane<T> {
 
     /// Downscales the source plane by a factor of `scale`, writing the result to `in_plane` (not padded)
     ///
-    /// `in_plane`'s width and height must be sufficient for `scale`.
-    #[hawktracer(downscale_in_place)]
+    /// # Panics
+    ///
+    /// - If the current plane's width and height are not at least `SCALE` times the `in_plane`'s
+    #[profiling::function(downscale_in_place)]
     pub fn downscale_in_place<const SCALE: usize>(&self, in_plane: &mut Plane<T>) {
+        let stride = in_plane.cfg.stride;
+        let width = in_plane.cfg.width;
+        let height = in_plane.cfg.height;
+
+        if stride == 0 || self.cfg.stride == 0 {
+            panic!("stride cannot be 0");
+        }
+
+        assert!(width * SCALE <= self.cfg.stride - self.cfg.xorigin);
+        assert!(height * SCALE <= self.cfg.alloc_height - self.cfg.yorigin);
+
         // SAFETY: Bounds checks have been removed for performance reasons
         unsafe {
             let src = self;
@@ -623,19 +641,7 @@ impl<T: Pixel> Plane<T> {
             let half_box_pixels = box_pixels as u32 / 2; // Used for rounding int division
 
             let data_origin = src.data_origin();
-
-            let stride = in_plane.cfg.stride;
-            let width = in_plane.cfg.width;
-            let height = in_plane.cfg.height;
-
-            if stride == 0 {
-                debug_unreachable!();
-            }
-            if src.cfg.stride == 0 {
-                debug_unreachable!();
-            }
-
-            let plane_data_mut_slice = in_plane.data.deref_mut();
+            let plane_data_mut_slice = &mut *in_plane.data;
 
             // Iter dst rows
             for row_idx in 0..height {
